package com.jaworski.security.handbook;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class LetterCipherApplet extends Applet {
 // Define key
 private LetterSubstitution key = new LetterSubstitution();

 // Declare GUI elements
 private TextArea plainText = new TextArea(5,50);
 private TextArea cipherText = new TextArea(5,50);
 private Button keyButton = new Button("Select Key");
 private Button encrypt = new Button("Encrypt");
 private Button decrypt = new Button("Decrypt");
 private KeyDialog keyDialog = new KeyDialog();
 
 public LetterCipherApplet() {
  // Layout applet
  layoutApplet();
  // Add event handlers
  keyButton.addActionListener(new ButtonHandler());
  encrypt.addActionListener(new ButtonHandler());
  decrypt.addActionListener(new ButtonHandler());
  // Display applet
  resize(400,400);
 }
 private void layoutApplet() {  
  // All these panels are used so that the applet displays 
  // well under Navigator and Internet Explorer.
  Panel keyPanel = new Panel();
  keyPanel.add(keyButton);
  keyPanel.add(encrypt);
  keyPanel.add(decrypt);
  Panel textPanel = new Panel();
  Panel textPanel1 = new Panel();
  Panel textPanel2 = new Panel();
  textPanel1.add(new Label("     Plain Text:     "));
  textPanel1.add(plainText);
  textPanel2.add(new Label("     Cipher Text:     "));
  textPanel2.add(cipherText);
  textPanel.setLayout(new GridLayout(2,1));
  textPanel.add(textPanel1);
  textPanel.add(textPanel2);
  setLayout(new BorderLayout());
  add("North",keyPanel);
  add("Center",textPanel);
}
 class ButtonHandler implements ActionListener {
  // Handle the Encrypt and Decrypt buttons
  public void actionPerformed(ActionEvent e) {
   if(e.getActionCommand().equals(encrypt.getLabel())) 
    cipherText.setText(LetterCipher.encrypt(plainText.getText(),key));
   else if(e.getActionCommand().equals(decrypt.getLabel())) 
    plainText.setText(LetterCipher.decrypt(cipherText.getText(),key));
   else if(e.getActionCommand().equals(keyButton.getLabel())) 
    keyDialog.setVisible(true);
  }
 }
 class KeyDialog extends Frame {
  // GUI controls
  private Button ok = new Button("OK");
  private Button cancel = new Button("Cancel");
  private Label[] labels = new Label[NUM_LETTERS];
  private Choice[] choices = new Choice[NUM_LETTERS];
  
  // Defined for convenience
  private static final int NUM_LETTERS = 26;
  private final String[] alphabet = 
   {"A","B","C","D","E","F","G","H","I","J","K","L","M",
    "N","O","P","Q","R","S","T","U","V","W","X","Y","Z"};
  
  KeyDialog() {
   setTitle("Key Dialog");
   setup();
   pack();
   setBounds(50,50,400,400);
  }
  private void setup() {
   // Create and initialize labels
   for(int i=0;i<labels.length;++i) 
    labels[i] = new Label(alphabet[i]+": ");
   // Create and initialize choices
   for(int i=0;i<choices.length;++i) {
    choices[i] = new Choice();
    for(int j=0;j<alphabet.length;++j)
     choices[i].add(alphabet[j]);
    choices[i].select(i);
   }
   // Layout GUI
   Panel northPanel = new Panel();
   northPanel.add(ok);
   northPanel.add(cancel);
   Panel centerPanel = new Panel();
   centerPanel.setLayout(new GridLayout(13,7));
   for(int i=0;i<13;++i) {
    centerPanel.add(labels[i]);
    centerPanel.add(choices[i]);
    centerPanel.add(new Label("     "));
    centerPanel.add(new Label("     "));
    centerPanel.add(new Label("     "));
    centerPanel.add(labels[i+13]);
    centerPanel.add(choices[i+13]);
   }
   add("North",northPanel);
   add("Center",centerPanel);
   // Add event handlers
   ok.addActionListener(new ActionListener() {
    public void actionPerformed(ActionEvent e) {
     for(int i=0;i<alphabet.length;++i)
      key.mapTo(alphabet[i].charAt(0),
       choices[i].getSelectedItem().charAt(0));
     keyDialog.setVisible(false);
    }
   });
   cancel.addActionListener(new ActionListener() {
    public void actionPerformed(ActionEvent e) {
     keyDialog.setVisible(false);
    }
   });
  }
 }
}
