package com.jaworski.security.handbook;

import java.io.*;
import java.security.*;

public class SHA1ConsoleApp {
 private static int BUFFER_SIZE = 32*1024;
 public static void main(String[] args) {
  String help = "Usage: java com.jaworski.security.handbook.";
  help += "SHA1ConsoleApp files";
  if(args.length == 0) {
   System.out.println(help);
   return;
  }
  for(int i=0;i<args.length;++i) {
   // Computer message digest for files identified 
   // by each command line argument.
   System.out.println(args[i]+": "+md(args[i]));
  }
 }
 public static String md(String f) {
  // Compute message digest for file named by f.
  BufferedInputStream file;
  // Create an SHA-1 message digest
  MessageDigest md;
  DigestInputStream in;
  String digestString = "";
  try {
   // Open the file
   file = new BufferedInputStream(new FileInputStream(f));
   // Create an SHA-1 message digest
   md = MessageDigest.getInstance("SHA-1");
   // Filter the file as a DigestInputStream object
   in = new DigestInputStream(file,md);
   int i;
   byte[] buffer = new byte[BUFFER_SIZE];
   // Read the file and compute the digest
   do {
    i = in.read(buffer,0,BUFFER_SIZE);
   } while(i == BUFFER_SIZE);
   // Get the final digest and convert it to a String.
   md = in.getMessageDigest();
   in.close();
   byte[] digest = md.digest();
   digestString = Conversion.byteArrayToHexString(digest);
  }catch (FileNotFoundException e) {
   return f+" not found.";
  }catch (NoSuchAlgorithmException e) {
    return "SHA-1 not supported.";
  }catch (IOException e) {
   return "Error reading from "+f+".";  
  }
  // Return the final digest as a String.
  return digestString;
 }
}
