package com.jaworski.security.handbook;

import java.math.*;
import java.security.*;
import java.security.spec.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;

public class KeyAgreementApp { 
 // SKIP base and modulus values are represented by a 
 // DHParameterSpec object
 static DHParameterSpec skipParameterSpec;
 public static void main(String[] args) throws Exception {
  // Create common base and modulus used with SKIP
  createBaseAndModulus();
  // Generate key pair for 1
  System.out.println("1 is generating a key pair");
  KeyPairGenerator kpg1 = KeyPairGenerator.getInstance("DH");
  kpg1.initialize(skipParameterSpec);
  KeyPair kp1 = kpg1.generateKeyPair();
  // Create a KeyAgreement object using the private key
  System.out.println("1 is creating a key agreement object");
  KeyAgreement ka1 = KeyAgreement.getInstance("DH");
  DHPrivateKey privateKey1 = (DHPrivateKey) kp1.getPrivate();
  DHPublicKey publicKey1 = (DHPublicKey) kp1.getPublic();
  ka1.init(privateKey1);
  System.out.println("1 is using "+publicKey1.getY()+
   " for its public key");
  // Generate key pair for 2
  System.out.println("2 is generating a key pair");
  KeyPairGenerator kpg2 = KeyPairGenerator.getInstance("DH");
  kpg2.initialize(skipParameterSpec);
  KeyPair kp2 = kpg2.generateKeyPair();
  // Create a KeyAgreement object using the private key
  System.out.println("2 is creating a key agreement object");
  KeyAgreement ka2 = KeyAgreement.getInstance("DH");
  DHPrivateKey privateKey2 = (DHPrivateKey) kp2.getPrivate();
  DHPublicKey publicKey2 = (DHPublicKey) kp2.getPublic();
  ka2.init(privateKey2);
  System.out.println("2 is using "+publicKey2.getY()+
   " for its public key");
  // Use the KeyAgreement object of 1 to generate its shared key
  ka1.doPhase(publicKey2,true);
  byte[] sharedKey1 = ka1.generateSecret();
  System.out.println("1 is using "+new BigInteger(1,sharedKey1)+
   " for its shared key");
  // Use the KeyAgreement object of 2 to generate its shared key
  ka2.doPhase(publicKey1,true);
  byte[] sharedKey2 = ka2.generateSecret();
  System.out.println("2 is using "+new BigInteger(1,sharedKey2)+
   " for its shared key");
 }
 static void createBaseAndModulus() {
  // The SKIP modulus value
  String s = "F488FD584E49DBCD20B49DE49107366B336C380D451D0F7C88" +
   "B31C7C5B2D8EF6F3C923C043F0A55B188D8EBB558CB85D38D3" +
   "34FD7C175743A31D186CDE33212CB52AFF3CE1B1294018118D" +
   "7C84A70A72D686C40319C807297ACA950CD9969FABD00A509B" +
   "0246D3083D66A45D419F9C7CBD894B221926BAABA25EC355E9" +
   "2F78C7";
  BigInteger base = BigInteger.valueOf(2);
  BigInteger modulus = new BigInteger(s,16);
  skipParameterSpec = new DHParameterSpec(modulus,base);
 }
}
