package com.jaworski.security.handbook;
import java.awt.*;
import java.awt.event.*;
import java.security.*;
import javax.crypto.*;
import javax.crypto.spec.*;

public class PBEApp extends Frame { 
 private int WIDTH = 600;
 private int HEIGHT = 500;
 private TextField inputTF;
 private TextArea outputTA;
 public static void main(String[] args) {
  new PBEApp();
 }
 public PBEApp() {
  super("PBEApp");
  setup();
  pack();
  setSize(WIDTH,HEIGHT);
  addWindowListener(new WindowHandler());
  setVisible(true);
 }
 void setup() {
  // Create panels for the input and output text fields
  Panel inputPanel = new Panel();
  Panel outputPanel = new Panel();
  Panel buttonPanel = new Panel();
  inputPanel.setLayout(new GridLayout(3,1));
  outputPanel.setLayout(new GridLayout(1,1));
  Label inputLabel = new Label("Enter pass phrase:");
  Button generateButton = new Button("Generate Key");
  generateButton.addActionListener(new ButtonHandler());
  inputTF = new TextField(70);
  outputTA = new TextArea(10,70);
  inputTF.setFont(new Font("Monospaced",Font.PLAIN,14));
  outputTA.setFont(new Font("Monospaced",Font.PLAIN,14));
  // Add the components to the panels
  inputPanel.add(inputLabel);
  inputPanel.add(inputTF);
  buttonPanel.add(generateButton);  
  inputPanel.add(buttonPanel);
  outputPanel.add(outputTA);  
  // Add to the frame
  add("North",inputPanel);
  add("Center",outputPanel);
 }
 private String generatePBEKey(char[] charArray) {
  String s = "";
  try {
   // Use the char array to create a PBEKeySpec
   PBEKeySpec keySpec = new PBEKeySpec(charArray);
   // Create a SecretKeyFactory for the PBE key
   SecretKeyFactory keyFactory = 
    SecretKeyFactory.getInstance("PBEWithMD5AndDES");
   // Generate the key from the key spec
   SecretKey key = keyFactory.generateSecret(keySpec);
   // Convert the key to a byte buffer
   byte[] keyBuffer = key.getEncoded();
   // Convert the byte buffer to a Base64 string
   s = Conversion.byteArrayToBase64String(keyBuffer);
  }catch(Exception e) {
   s = e.toString();
  }
  // Return the key as a string or 
  // any exception that was generated.
  return s;
 }
 class ButtonHandler implements ActionListener {
  public void actionPerformed(ActionEvent e) {
   // Generate the PBE key and put it in the text area
   outputTA.setText(generatePBEKey(inputTF.getText().toCharArray()));
  }
 }
 class WindowHandler extends WindowAdapter {
  public void windowClosing(WindowEvent e) {
   setVisible(false);
   dispose();
   System.exit(0);
  }
 }
}
