package com.jaworski.security.handbook;

import java.security.*;
import java.security.spec.*;
import javax.crypto.*;

public class Rot13Cipher extends CipherSpi {
 private int opmode = Cipher.ENCRYPT_MODE;
 private int encryptionKey = 13;
 private int decryptionKey = 243;
 // Default constructor.
 public Rot13Cipher() {
 }
 // Implement the cipher
 private byte cipher(byte b) {
  int key = encryptionKey;
  if(opmode == Cipher.DECRYPT_MODE)
   key = decryptionKey;
  return (byte) ((b + key) % 256);
 }
 // CipherSpi that must be implemented.
 protected void engineInit(int opmode, Key key, 
   AlgorithmParameterSpec params, SecureRandom random) {
  this.opmode = opmode;
 }
 protected void engineInit(int opmode, Key key, 
   AlgorithmParameters params, SecureRandom random) {
  this.opmode = opmode;
 }
 protected void engineInit(int opmode, Key key, SecureRandom random) {
  this.opmode = opmode;
 }
 protected byte[] engineUpdate(byte[] input, int inputOffset, 
   int inputLen) {
  byte[] b = new byte[inputLen];
  for(int i=0; i<inputLen; ++i) 
   b[i] = cipher(input[inputOffset + i]);
  return b;
 }
 protected int engineUpdate(byte[] input, int inputOffset, 
   int inputLen, byte[] output, int outputOffset)
   throws ShortBufferException {
  String msg = "Not enough room in output buffer.";
  if((output.length - outputOffset) < inputLen) 
   throw new ShortBufferException(msg);
  for(int i=0; i<inputLen; ++i) 
   output[outputOffset + i] = cipher(input[inputOffset + i]);
  return inputLen;
 }
 protected byte[] engineDoFinal(byte[] input, int inputOffset, 
   int inputLen) {
  byte[] b = new byte[inputLen];
  for(int i=0; i<inputLen; ++i) 
   b[i] = cipher(input[inputOffset + i]);
  return b;
 }
 protected int engineDoFinal(byte[] input, int inputOffset, 
   int inputLen, byte[] output, int outputOffset)
   throws ShortBufferException {
  String msg = "Not enough room in output buffer.";
  if((output.length - outputOffset) < inputLen) 
   throw new ShortBufferException(msg);
  for(int i=0; i<inputLen; ++i) 
   output[outputOffset + i] = cipher(input[inputOffset + i]);
  return inputLen;
 } 
 protected int engineGetBlockSize() {
  return 1;
 }
 protected int engineGetOutputSize(int inputLen) {
  return inputLen;
 }
 protected void engineSetMode(String mode) 
   throws NoSuchAlgorithmException {
  String msg = "ROT13 only supports ECB mode.";
  if(!mode.toUpperCase().equals("ECB")) 
   throw new NoSuchAlgorithmException(msg);
 }
 protected void engineSetPadding(String padding)
   throws NoSuchPaddingException {
  String msg = "ROT13 only supports the NoPadding option.";
  if(!padding.toUpperCase().equals("NOPADDING")) 
   throw new NoSuchPaddingException(msg);
 }
 protected AlgorithmParameters engineGetParameters() {
  return null;
 }
 protected byte[] engineGetIV() {
  return null;
 }
}
